<?php

namespace Drupal\image_style_generate\Plugin\migrate\source;

use Drupal\Component\Utility\NestedArray;
use Drupal\image_style_generate\ImageStyleRow;
use Drupal\migrate\MigrateException;
use Drupal\migrate\Plugin\migrate\source\SourcePluginBase;
use Drupal\migrate\Plugin\MigrationInterface;

/**
 * Migrate source plugin for generating image styles.
 *
 * @MigrateSource(
 *   id = "image_style_generate"
 * )
 */
class ImageStyleGenerate extends SourcePluginBase {

  /**
   * Data rows.
   *
   * @var array[]
   */
  protected $dataRows = [];

  /**
   * List of available source fields.
   *
   * Keys are the field machine names as used in field mappings, values are
   * descriptions.
   *
   * @var array
   */
  protected $fields = [];

  /**
   * List of key fields, as indexes.
   *
   * @var array
   */
  protected $keys = [];

  /**
   * List of IDs.
   *
   * @var array
   */
  protected $ids = [];

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);

    if (empty($this->configuration['ids'])) {
      throw new MigrateException('You must declare "ids" as a unique array of fields in your source settings.');
    }

    $this->dataRows = $this->generateRows();
    $this->ids = $configuration['ids'];
  }

  /**
   * {@inheritdoc}
   */
  public function __toString() {
    $groups = array_keys($this->configuration['style_groups']);
    return implode(', ', $groups);
  }

  /**
   * {@inheritdoc}
   */
  public function initializeIterator() {
    return new \ArrayIterator($this->dataRows);
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return $this->ids;
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    return empty($this->configuration['fields']) ? [] : $this->configuration['fields'];
  }

  /**
   * Generate rows of image styles to create.
   *
   * @var array $image_style_data
   *   The data to use to generate the rows.
   *
   * @return array
   *   The rows generated by the image style data.
   */
  protected function generateRows() {
    $rows = [];

    foreach ($this->styleGroups() as $group_id => $group) {
      $base_sizes = $this->baseSizes($group_id);

      foreach (array_keys($base_sizes) as $base_size) {
        $size_scales = $this->sizeScales($group_id, $base_size);

        foreach (array_keys($size_scales) as $size_scale) {
          $effects = $this->effects($group_id, $base_size, $size_scale);

          $data = [
            'size_multiplier' => $this->styleGroupConfig($group_id, 'size_multiplier'),
            'id_pattern' => $this->styleGroupConfig($group_id, 'id_pattern'),
            'label_pattern' => $this->styleGroupConfig($group_id, 'label_pattern'),
            'group_id' => $group_id,
            'group_label' => $group['label'],
            'base_size_dimension' => $group['base_size_dimension'],
            'aspect_ratio' => $group['aspect_ratio'],
            'base_size' => $base_size,
            'size_scale' => $size_scale,
            'effects' => $effects,
          ];

          $image_style_row = new ImageStyleRow($data);

          $rows[$image_style_row->getId()] = $image_style_row->getRow();
        }
      }
    }

    return $rows;
  }

  /**
   * Get the style groups.
   *
   * @return array
   *   The style groups.
   */
  protected function styleGroups() {
    if (!isset($this->configuration['style_groups'])) {
      throw new MigrateException('Style groups must be defined in the migration configuration.');
    }

    $style_groups = $this->configuration['style_groups'];

    foreach ($style_groups as $id => $config) {
      if (!isset($config['label'])) {
        $style_groups[$id]['label'] = NULL;
      }

      if (!isset($config['base_size_dimension'])) {
        $style_groups[$id]['base_size_dimension'] = NULL;
      }

      if (!isset($config['aspect_ratio'])) {
        $style_groups[$id]['aspect_ratio'] = NULL;
      }
    }

    return $style_groups;
  }

  /**
   * Get the base sizes for a given style group.
   *
   * @var string $group_id
   *   The ID of the style group.
   *
   * @return array
   *   The base sizes for the style group.
   */
  protected function baseSizes($group_id) {
    $arrays_to_merge = [
      $this->configSection('base_sizes', ['defaults']),
      $this->configSection('base_sizes', ['style_groups', $group_id]),
    ];

    // Get all of the base sizes.
    $base_sizes_merged = NestedArray::mergeDeepArray($arrays_to_merge, TRUE);

    $base_sizes = [];

    foreach ($base_sizes_merged as $base_size => $config) {
      // Skip any items that are set to "false".
      if ($config === FALSE) {
        continue;
      }

      // If $config is a scalar, assume it's the base size value.
      if (is_scalar($config)) {
        $base_size = (string) $config;
        $config = [];
      }

      if (!is_numeric($base_size) || $base_size < 1) {
        throw new MigrateException('A positive base size value must be defined in each base size definition.');
      }

      $base_sizes[$base_size] = $config;
    }

    return $base_sizes;
  }

  /**
   * Get the size scales for a given style group and base size.
   *
   * @var string $group_id
   *   The ID of the style group.
   * @var string $base_size
   *   The base size.
   *
   * @return array
   *   The size scales.
   */
  protected function sizeScales($group_id, $base_size) {
    $arrays_to_merge = [
      $this->configSection('size_scales', ['defaults']),
      $this->configSection('size_scales', [
        'defaults',
        'base_sizes',
        $base_size,
      ]),
      $this->configSection('size_scales', ['style_groups', $group_id]),
      $this->configSection('size_scales', [
        'style_groups',
        $group_id,
        'base_sizes',
        $base_size,
      ]),
    ];

    // Get all of the size scales.
    $size_scales_merged = NestedArray::mergeDeepArray($arrays_to_merge, TRUE);

    $size_scales = [];

    foreach ($size_scales_merged as $size_scale => $config) {
      // Remove any items set to "false".
      if ($config === FALSE) {
        continue;
      }

      // If $config is a scalar, assume it's the size scale value.
      if (is_scalar($config)) {
        $size_scale = $config;
        $config = [];
      }

      if (!is_numeric($size_scale) || $size_scale < 1) {
        throw new MigrateException('A positive size scale value must be defined in each size scale definition.');
      }

      $size_scales[$size_scale] = $config;
    }

    return $size_scales;
  }

  /**
   * Get the effects to be applied for a given configuration set.
   *
   * @var string $group_id
   *   The ID of the style group.
   * @var string $base_size
   *   The base size.
   * @var string $size_scale
   *   The size scale.
   *
   * @return array
   *   The effects for the given configuration set.
   */
  protected function effects($group_id, $base_size, $size_scale) {
    $arrays_to_merge = [
      $this->configSection('effects', ['defaults']),
      $this->configSection('effects', [
        'defaults',
        'base_sizes',
        $base_size,
      ]),
      $this->configSection('effects', [
        'defaults',
        'size_scales',
        $size_scale,
      ]),
      $this->configSection('effects', [
        'defaults',
        'base_sizes',
        $base_size,
        'size_scales',
        $size_scale,
      ]),
      $this->configSection('effects', ['style_groups', $group_id]),
      $this->configSection('effects', [
        'style_groups',
        $group_id,
        'base_sizes',
        $base_size,
      ]),
      $this->configSection('effects', [
        'style_groups',
        $group_id,
        'size_scales',
        $size_scale,
      ]),
      $this->configSection('effects', [
        'style_groups',
        $group_id,
        'base_sizes',
        $base_size,
        'size_scales',
        $size_scale,
      ]),
    ];

    // Get all of the effects.
    $effects_merged = NestedArray::mergeDeepArray($arrays_to_merge, TRUE);

    $effects = [];

    // Remove any items set to "false".
    foreach ($effects_merged as $id => $value) {
      if ($value === FALSE) {
        continue;
      }

      $effects[$id] = $value;
    }

    return $effects;
  }

  /**
   * Get a configuration value for a style group or fall back to the default.
   *
   * @var string $config_id
   *   The ID of the configuration value to retrieve.
   *
   * @return mixed
   *   The configuration value.
   */
  protected function styleGroupConfig($group_id, $config_id) {
    if (isset($this->configuration['style_groups'][$group_id][$config_id])) {
      $value = $this->configuration['style_groups'][$group_id][$config_id];
    }
    elseif (isset($this->configuration['defaults'][$config_id])) {
      $value = $this->configuration['defaults'][$config_id];
    }
    else {
      $value = NULL;
    }

    return $value;
  }

  /**
   * Return a configuration section.
   *
   * @var string $section
   *   The key of the section to retrieve.
   * @var array $parents
   *   An array of parent keys, starting with the outermost key.
   *
   * @return array
   *   The section configuration or empty if undefined.
   *
   * @see NestedArray::getValue()
   */
  protected function configSection($section, array $parents = []) {
    $keys = $parents;
    array_push($keys, $section);

    $config_section = NestedArray::getValue($this->configuration, $keys);

    if (!$config_section) {
      return [];
    }
    else {
      return $config_section;
    }
  }

}
